/// @addtogroup dynsys2
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BasicTaylor.h
///
/// @author Daniel Wilczak, Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2008 by the CAPD Group.
//
// Distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.

/* Authors: Daniel Wilczak, 2001-2004, Tomasz Kapela, 2008 */



#ifndef _CAPD_DYNSYS2_BASICTAYLOR_H_ 
#define _CAPD_DYNSYS2_BASICTAYLOR_H_ 

#include <string>

namespace capd{
  namespace dynsys2{
    
    /* MapT constraints:
     *  type definitions:
     *   - FunctionType
     *   - MatrixType
     *   - MatrixType::RowVectorType
     *   - MatrixType::ScalarType
     *
     *  methods:
     *   - getDimension         - dimension of the space
     *   - getOrder, setOrder   - order of Taylor expansion
     *   - operator()(a, i)     - computes coeffs a_i from a = a_{i-1}
     *   - variational(x_coeff, m_F, m_order);  
     *
     */
    
    template<typename MapT>
    class BasicTaylor {
      
    public:
      typedef MapT MapType;
      typedef typename MapT::FunctionType FunctionType;
      typedef typename MapType::MatrixType MatrixType;
      typedef typename MatrixType::RowVectorType VectorType;
      typedef typename MatrixType::ScalarType ScalarType;
      
      
      // Constr & Destr
      BasicTaylor(MapType & field, int order, const ScalarType & step);
      BasicTaylor(const BasicTaylor &);
      virtual ~BasicTaylor();
      BasicTaylor & operator=(const BasicTaylor & t);
      
      /// Computes image of vector v after one time step
      VectorType operator()(const VectorType & v) const;

      /// Computes image of vector v and derivatives with respect to initial conditions
      VectorType operator()(const VectorType &, MatrixType &derivative) const;

      // get & set functions for vector field
      const MapType& getField() const;              ///< Returns vector field
      MapType& getField();                          ///< Returns vector field
      virtual void setField(const MapType& Field);  ///< Sets new vector field
                   
      // get & set functions for Taylor method order
      int getOrder() const;                        ///< Returns Taylor method order 
      virtual void setOrder(int newOrder);         ///< Sets Taylor method order

      // get & set functions for time step
      const ScalarType& getStep() const;           ///< Returns current time step
                                                                                                                                                                                                                                                                                                                                                                
      void setStep(const ScalarType& newStep);     ///< Sets time step

      /// returns coefficients of Taylor expansion of the solution of ODE
      VectorType *getCoeff();
      
      /// returns coefficients of Taylor expansion of the monodromy matrix 
      MatrixType *getMatrixCoeff();
      
    protected:
      
      MapType & m_vField;              ///< vector field
      int m_order;                     ///< order of Taylor method
      ScalarType m_step;               ///< time step
      VectorType *m_coeff;             ///< coefficients of Taylor expansion of solution
      MatrixType *m_matrixCoeff;       ///< coefficients of Taylor expansion of variational part
      int m_dimension;                 ///< dimension of a vector space
      
      MatrixType *m_F;                 ///< temp table used in the computations of variational part
      void computeMatrixCoeff(int, const MatrixType& H) const;
      
    };
    
// -----------------------------------------------------------------------------
    
    template<typename MapType>
    inline const MapType& BasicTaylor<MapType>::getField() const {
      return m_vField;
    }
    
    template<typename MapType>
    inline MapType & BasicTaylor<MapType>::getField() {
      return m_vField;
    }
    
    template<typename MapType>
    inline int BasicTaylor<MapType>::getOrder() const {
      return m_order;
    }
    
    template<typename MapType>
    inline const typename BasicTaylor<MapType>::ScalarType& BasicTaylor<MapType>::getStep() const {
      return m_step;
    }
    
    template<typename MapType>
    inline void BasicTaylor<MapType>::setStep(const ScalarType& newStep) {
      m_step = newStep;
    }
    
    template<typename MapType>
    inline typename BasicTaylor<MapType>::VectorType* BasicTaylor<MapType>::getCoeff() {
      return m_coeff;
    }
    
    template<typename MapType>
    inline typename BasicTaylor<MapType>::MatrixType* BasicTaylor<MapType>::getMatrixCoeff() {
      return m_matrixCoeff;
    }
    
  }} // namespace capd::dynsys

#endif // _CAPD_DYNSYS2_BASICTAYLOR_H_ 

/// @}
