/// @addtogroup dynsys2
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file BasicTaylor.hpp
///
/// @author Daniel Wilczak
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2008 by the CAPD Group.
//
// Distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details.

/* Authors: Daniel Wilczak, 2001-2004, Tomasz Kapela, 2008 */


#ifndef _CAPD_DYNSYS2_BASICTAYLOR_HPP_ 
#define _CAPD_DYNSYS2_BASICTAYLOR_HPP_ 

#include <sstream>
#include <string>
#include <stdexcept>

#include "capd/factrial.h"
#include "dynsys2/BasicTaylor.h"

namespace capd{
  namespace dynsys2{
    
//-----------------------------------------------------------//
    /** 
     *  Constructor
     *
     *  @param[in] vectorField   vector field (right hand side of ODE)
     *  @param[in] order         order of Taylor expansion
     *  @param[in] step          time step for ODE integration
     *
     */
    template<typename MapType>
    BasicTaylor<MapType>::BasicTaylor(
            MapType & vectorField,
            int order,
            const ScalarType& step
    ) : m_vField(vectorField), m_order(order), m_step(step) {
      
      m_dimension = m_vField.getDimension();
      m_vField.setOrder(m_order+1);
      m_coeff = new (m_dimension) VectorType[m_order+2];
      m_matrixCoeff = new (m_dimension, m_dimension) MatrixType[m_order+2];
      m_F = new (m_dimension, m_dimension) MatrixType[m_order+2];
    }
    
//-----------------------------------------------------------//
    /**
     *  Copying constructor
     */
    template<typename MapType>
    BasicTaylor<MapType>::BasicTaylor(
       const BasicTaylor &t
    ) : m_vField(t.m_vField), m_order(t.m_order), m_step(t.m_step),  m_dimension(t.m_dimension) {
      
      m_coeff = new (m_dimension) VectorType[m_order+2];
      m_matrixCoeff = new (m_dimension, m_dimension) MatrixType[m_order+2];
      m_F = new (m_dimension, m_dimension) MatrixType[m_order+2];
      
      for(int i=0; i<m_order+2; ++i) {
        m_coeff[i] = t.m_coeff[i];
        m_matrixCoeff[i] = t.m_matrixCoeff[i];
        m_F[i] = t.m_F[i];
      }
    }
    
//-----------------------------------------------------------//
    
    /**
     *  Destructor
     */
    template<typename MapType>
    BasicTaylor<MapType>::~BasicTaylor() {
      delete [] m_coeff;
      delete [] m_matrixCoeff;
      delete [] m_F;
    }
    
//-----------------------------------------------------------//
    
    template<typename MapType>
    BasicTaylor<MapType>& BasicTaylor<MapType>::operator=(const BasicTaylor& A_t) {
      if(this != &A_t){
        m_vField = A_t.m_vField;
        m_step = A_t.m_step;
        
        if((A_t.m_dimension != m_dimension) || (A_t.m_order != m_order)){
          
          VectorType * coeff = new (A_t.m_dimension) VectorType[A_t.m_order+2];
          MatrixType * matrixCoeff = new (A_t.m_dimension, A_t.m_dimension) MatrixType[A_t.m_order+2];
          MatrixType * F = new (A_t.m_dimension, A_t.m_dimension) MatrixType[A_t.m_order+2];
          
          for(int i=0; i<m_order+2; ++i) {
            coeff[i] = A_t.m_coeff[i];
            matrixCoeff[i] = A_t.m_matrixCoeff[i];
            F[i] = A_t.m_F[i];
          }
          delete[] m_coeff;           m_coeff = coeff;
          delete[] m_matrixCoeff;     m_matrixCoeff = matrixCoeff;
          delete[] m_F;               m_F = F;
          
        }else{
          for(int i=0; i<A_t.m_order+2; ++i) {
            m_coeff[i] = A_t.m_coeff[i];
            m_matrixCoeff[i] = A_t.m_matrixCoeff[i];
            m_F[i] = A_t.m_F[i];
          }
        }
        m_order = A_t.m_order;
        m_dimension = A_t.m_dimension;
      }
      return *this;
    }
    
//-----------------------------------------------------------//
    
    
    template<typename MapType>
    void BasicTaylor<MapType>::setOrder(int newOrder) {
      if(newOrder!=m_order) {
        m_order = newOrder;
        m_vField.setOrder(m_order+1);
        
        delete [] m_coeff;
        delete [] m_matrixCoeff;
        delete [] m_F;
        m_coeff = new (m_dimension) VectorType[m_order+2];
        m_matrixCoeff = new (m_dimension, m_dimension) MatrixType[m_order+2];
        m_F = new (m_dimension, m_dimension) MatrixType[m_order+2];
      }
    }
    
    
//-----------------------------------------------------------//
    
    
    template<typename MapType>
    void BasicTaylor<MapType>::setField(const MapType& vectorField) {
      m_vField = vectorField;
      m_vField.setOrder(m_order+1);
      if(m_dimension!=m_vField.getDimension()) {
        delete [] m_coeff;
        delete [] m_matrixCoeff;
        delete [] m_F;
        m_dimension = m_vField.getDimension();
        m_coeff = new (m_dimension) VectorType[m_order+2];
        m_matrixCoeff = new (m_dimension, m_dimension) MatrixType[m_order+2];
        m_F = new (m_dimension, m_dimension) MatrixType[m_order+2];
      }
    }
    
//-----------------------------------------------------------//
    /**
     *  Computes coefficients of Taylor expansion of monodromy matrix
     *  results are stored in m_matrixCoeff
     *
     *  @param[in]  order  to which order we should compute coefficients
     *  @param[in]  V      initial value of the monodromy matrix
     *
     *  @warning  m_coeff need to be computed first
     *  @warning  in previous implementations we also assumed that m_F is already computed
     *            now we compute it everytime computeMatrixCoeff is called.
     */
    template<typename MapType>
    void BasicTaylor<MapType>::computeMatrixCoeff(int order, const MatrixType &V) const {
      if(order<0) {
        std::ostringstream message;
        message << "Taylor2 error: negative index " << order << " of Taylor2 coefficient in function computeMatrixCoeff";
        throw std::out_of_range(message.str());
      }
      
      // now we compute the coefficients for the Jacobian matrix
      // the coefficients will be stored in m_F
      m_vField.variational(m_coeff, m_F, order+1);
      
      m_matrixCoeff[0] = V;
      
      int p, i, j, k, r;
      for(p=0;  p<=order; ++p) {
        typename MatrixType::ContainerIterator b = m_matrixCoeff[p+1].cbegin();
        for(i=1; i<=m_dimension; ++i)
          for(j=1; j<=m_dimension; ++j) {
            ScalarType result(0);
            for(k=1; k<=m_dimension; ++k)
              for(r=0; r<=p; ++r)
                result += m_F[r](i, k) * m_matrixCoeff[p-r](k, j);
            *b = result/(p+1);
            ++b;
          }
      }
    }
    
//-----------------------------------------------------------//
    /**
     *  Computes an image of a set v after one time step
     *
     *  As a byproduct it also computes m_coeff
     *  coefficients of the Taylor expansion of the solution
     *
     *  @param[in]  v  set of initial conditions
     *  @returns    positions after one time step
     */
    
    template<typename MapType>
    typename BasicTaylor<MapType>::VectorType BasicTaylor<MapType>::operator()(const VectorType &v) const {
      m_coeff[0] = v;        //  m_coeff will contain Taylor expansion coefficients in point <v>
      // now we initiate 0-th order term
      int r;
      for(r=0; r<m_order; ++r)                         // recurrent computation of consequtive
        m_coeff[r+1] = m_vField(m_coeff[r], r)/(r+1);  // Taylor coefficients
      
      // sumation of Taylor series
      VectorType result = m_coeff[m_order];
      for(r=m_order-1; r>=0; --r)
        result = result*m_step + m_coeff[r];
      return result;
    }
    
//-----------------------------------------------------------//
    /**
     *  Computes a solution and its derivatives with respect to initial conditions after one time step
     *
     *  As a byproduct it also computes
     *  coefficients of the Taylor expansion of the solution and derivative
     *  and stores them in m_coeff and m_matrixCoeff
     *
     *  @param[in]  v    set of initial conditions
     *  @param[out] der  derivatives of solution
     *  @returns    positions after one time step
     */
    
    template<typename MapType>
    typename BasicTaylor<MapType>::VectorType BasicTaylor<MapType>::operator()(
            const VectorType & v,
            MatrixType & der
    ) const {
      
      // first we compute ceofficients for x
      VectorType result = (*this)(v);
      
      // this function computes coefficients up to index 'order'
      // and stores them in matrixCoeff
      computeMatrixCoeff(m_order-1, MatrixType::Identity(m_dimension));
      
      // the sumation of the Taylor series
      der = m_matrixCoeff[m_order];
      int r;
      for(r=m_order-1; r>=0; --r)
        der = der*m_step + m_matrixCoeff[r];
      
      return result;
    }
    
    
  }} //namespace capd::dynsys2

#endif // _CAPD_DYNSYS2_BASICTAYLOR_HPP_ 

/// @}
