// This is WORDS.H. Copyright (C) 2002-2003 by Pawel Pilarczyk.
// It contains the data structure for storing words (strings).
// This is free software. No warranty. Consult 'license.txt' for details.
// Started on May 5, 2002. Last revision: February 22, 2003.


#ifndef _WORDS_H_
#define _WORDS_H_

#include "options.h"
#include "integer.h"
#include "textfile.h"
#include "hashsets.h"

#include <cstdlib>
#include <cstring>
#include <iostream>
#include <fstream>
#include <sstream>


// the data types defined within this file
class word;
typedef hashedset<word> words;


//Doc//
// --------------------------------------------------
// ---------------------- word ----------------------
// --------------------------------------------------
// A word (just a string).

class word
{
public:
	// various constructors
	word ();
	word (const char *s);
	word (const word &w);

	// destructor
	~word ();

	// the assignment operator
	word &operator = (const word &w);

	// return the length of the word (without '\0')
	int length () const;

	// return a pointer to the contents of the word
	operator const char * () const;

	// concatenation operators
	word &operator += (const word &w);
	word operator + (const word &w) const;

	// return the value of the number contained in the word;
	// allow a preceding '+'
	operator int () const;

	// two hashing keys required by the hashing set template
	unsigned hashkey1 () const;
	unsigned hashkey2 () const;

protected:
	// the length of the word (without '\0')
	int len;

	// a table containing the word
	char *txt;

}; /* class word */
//End//

// --------------------------------------------------

inline word::word ()
{
	len = 0;
	txt = NULL;
	return;
} /* word::word */

inline word::word (const char *s)
{
	len = s ? strlen (s) : 0;
	if (!len)
		txt = NULL;
	else
	{
		txt = new char [len + 1];
		if (!txt)
			throw "Not enough memory to create a word.";
		strcpy (txt, s);
	}
	return;
} /* word::word */

inline word::word (const word &w)
{
	len = w. len;
	if (!len)
		txt = NULL;
	else
	{
		txt = new char [len + 1];
		if (!txt)
			throw "Not enough memory to copy a word.";
		strcpy (txt, w. txt);
	}
	return;
} /* word::word */

inline word::~word ()
{
	if (txt)
		delete [] txt;
	return;
} /* word::~word */

inline word &word::operator = (const word &w)
{
	if (txt)
		delete [] txt;
	len = w. len;
	if (w. txt)
	{
		txt = new char [len + 1];
		if (!txt)
			throw "Not enough memory to copy a word.";
		strcpy (txt, w. txt);
	}
	else
		txt = NULL;
	return *this;
} /* operator = */

inline int word::length () const
{
	return len;
} /* length */

inline word::operator const char * () const
{
	return txt;
} /* operator int */

inline word &word::operator += (const word &w)
{
	if (!len)
	{
		*this = w;
		return *this;
	}
	if (!w. len)
		return *this;
	int newlen = len + w. len;
	char *newtxt = new char [newlen + 1];
	strcpy (newtxt, txt);
	strcat (newtxt, w. txt);
	delete [] txt;
	txt = newtxt;
	len = newlen;
	return *this;
} /* operator += */

inline word word::operator + (const word &w) const
{
	word new_w (*this);
	new_w += w;
	return new_w;
} /* operator + */

inline word::operator int () const
{
	if (!len)
		return 0;
	char *s = txt;
	if (*s == '+')
		s ++;
	int num;
	std::istringstream str (s);
	str >> num;
	if (!str)
		return 0;
	else
		return num;
} /* operator int */

inline unsigned word::hashkey1 () const
{
	if (!len)
		return 13;
	unsigned code = ((unsigned) (txt [0]) << 6) ^
		((unsigned) (txt [len - 1]));
	if (len > 3)
		code ^= (unsigned) (txt [2]) << 4;
	return code;
} /* word::hashkey1 */

inline unsigned word::hashkey2 () const
{
	if (!len)
		return 7;
	unsigned code = ((unsigned) (txt [0])) ^
		((unsigned) (txt [len - 1] << 9));
	if (len > 4)
		code ^= (unsigned) (txt [3]) << 4;
	return code;
} /* word::hashkey2 */

//Doc//
// --------------------------------------------------

inline int operator == (const word &w1, const word &w2)
//End//
{
	if (w1. length () != w2. length ())
		return 0;
	if (!w1. length ())
		return 1;
	return !strcmp ((const char *) w1, (const char *) w2);
} /* operator == */

//Doc//
inline int operator != (const word &w1, const word &w2)
//End//
{
	return !(w1 == w2);
} /* operator != */

//Doc//
inline int operator == (const word &w, const char *c)
//End//
{
	if (!w. length ())
		return (!c || !*c);
	return !strcmp ((const char *) w, c);
} /* operator == */

//Doc//
inline int operator != (const word &w, const char *c)
//End//
{
	return !(w == c);
} /* operator != */

//Doc//
inline int operator == (const char *c, const word &w)
//End//
{
	return (w == c);
} /* operator == */

//Doc//
inline int operator != (const char *c, const word &w)
//End//
{
	return (w != c);
} /* operator != */

//Doc//
// --------------------------------------------------

// alphabetical comparison of words
inline int operator < (const word &w1, const word &w2)
//End//
{
	const char *c1 = (const char *) w1;
	const char *c2 = (const char *) w2;

	while (*c1 && *c2 && ((*c1) == (*c2)))
	{
		c1 ++;
		c2 ++;
	}
	return ((*c1) < (*c2));
} /* operator < */

//Doc//
inline int operator > (const word &w1, const word &w2)
//End//
{
	return (w2 < w1);
} /* operator > */

//Doc//
inline int operator <= (const word &w1, const word &w2)
//End//
{
	return !(w1 > w2);
} /* operator <= */

//Doc//
inline int operator >= (const word &w1, const word &w2)
//End//
{
	return !(w1 < w2);
} /* operator >= */


//Doc//
// --------------------------------------------------

// Append the string value of the given element to the word.
template <class type>
word &operator << (word &w, const type &elem)
//End//
{
	std::ostringstream s;
	s << elem;
	w += s. str (). c_str ();
	return w;
} /* operator << */


//Doc//
// --------------------------------------------------

// Write the given word to the output stream.
inline std::ostream &operator << (std::ostream &out, const word &w)
//End//
{
	if (w. length ())
		out << (const char *) w;
	return out;
} /* operator << */

//Doc//
// Read a word from the input stream.
inline std::istream &operator >> (std::istream &in, word &w)
//End//
{
	char buf [256];
	in >> buf;
	if (!!in)
		w = word (buf);
	return in;
} /* operator >> */


#endif // _WORDS_H_

